/* 
 *  Sursa pentru citirea joysticului 
 * https://lastminuteengineers.com/joystick-interfacing-arduino-processing/#:~:text=To%20read%20whether%20the%20joystick,GND%20terminal%20of%20the%20Arduino.&text=That's%20it.
 */
const int X_js_pin = 54;  // pinul de input pe OX a joystickului
const int Y_js_pin = 55;  // pinul de input pe OY a joystickului
const int SW_pin = 2;     // pinul de switch al joystickului

/* 
 *  Sursa lucru cu servo motoare
 * https://www.instructables.com/Arduino-Servo-Motors/
 */
#include <Servo.h>
int X_servo_pin = 6;  // pinul de output pentru servoul OX
int Y_servo_pin = 7;  // pinul de output pentru servoul OY
Servo ServoX, ServoY;

/*
 * Statusul Jocului
 */
int ongoing = 0;          // jocul se afla in derulare
int finished = 1;         // jocul este in asteptare
int game_state;           // starea jocului
long game_start_time;     // rezultatul functiei millis() la inceputul unui joc
int Game_status_pin = 8;  // pinul de input al senzorului de lumina

/*
 * Structura de date pentru salvarea coordonatelor pe OX si OY
 * Folosit atat pentru citirea inputului, cat si pentru scrierea sa
 */
typedef struct coordPair{
  // Pair values
  int OX, OY;

  // Pair constructors
  coordPair() : OX(0), OY(0) {}
  coordPair(int ox, int oy) : OX(ox), OY(oy) {}
} coordPair;

/*
 * Functie ce transfoma intervalul [0-1024] al inputului de la joystick
 * in intrevalul [0-30] de grade dorit
 */
coordPair transformCoords(coordPair in);

/*
 * Functie ce transmite outputul la servo motoare
 */
void writeOutput(coordPair out);

/*
 * Updateaza statusul jocului in functie de inputul senzorului de lumina
 */
void updateGameState();

void setup() {
  // joystick setup
  pinMode(SW_pin, INPUT);
  digitalWrite(SW_pin, HIGH);

  // servo setup
  ServoY.attach(Y_servo_pin);
  ServoX.attach(X_servo_pin);
  writeOutput(coordPair());

  // game-staet setup
  Serial.begin(9600);
  pinMode(Game_status_pin, INPUT);
  game_state = finished;
  game_start_time = 0;
}

void loop() {
  // citirea valorilor de la joystick
  coordPair joystick_in(analogRead(X_js_pin), analogRead(Y_js_pin));
  // transfomarea in coordonate pentru servo
  coordPair servo_out = transformCoords(joystick_in);
  // transmiterea valorilor pentru servo
  writeOutput(servo_out);
  // verificarea daca jocul s-a terminat sau a inceput un nou joc
  updateGameState();
}

coordPair transformCoords(coordPair in) {
  float factor = 30.0f / 1024.0f;
  int ox = factor * (float) in.OX;
  int oy = factor * (float) in.OY;
  return coordPair(ox, oy);
}

void writeOutput(coordPair out) {
  ServoX.write((out.OX > 30 || out.OX < 0) ? 0 : out.OX);
  ServoY.write((out.OY > 30 || out.OY < 0) ? 0 : out.OY);
}

void updateGameState() {
  int new_game_state = digitalRead(Game_status_pin);

  // jocul nu isi schimba starea
  if (game_state == new_game_state)
    return;

  // jocul trece din starea "ongoing" in starea "finished"
  if (game_state == ongoing && new_game_state == finished) {
    // timpul de la inceputul jocului la finalul acestuia
    long total_time = millis() - game_start_time;
    // buffer de 5 secunde de la inceputul jocului in care acesta nu se poate termina
    if (total_time < 5000) {
      game_start_time = millis();
      return;
    }

    // afisarea la serial a rezultatelor
    Serial.print("Game finished in ");
    Serial.print(total_time / 60000);
    Serial.print(" minutes ");
    Serial.print(total_time / 1000 - (total_time / 60000) * 60000);
    Serial.print(" seconds  and ");
    Serial.print(total_time % 1000);
    Serial.println(" milliseconds.");
    game_state = finished;
  }

  // jocul trece din starea "finished" in starea "ongoing"
  if (game_state == finished && new_game_state == ongoing) {
    Serial.println("New game");
    game_start_time = millis();
    game_state = ongoing;
  }
}
